#!/usr/bin/perl
#
#==============================================================================
#This file is part of CCTDE - Covert Channels and Tunnels Detection Engine v0.1
#(C) Simon Castro.
#CCTDE is free software; you can redistribute it  and/or  modify  it  under the
#terms of the GNU General Public License as published by the Free Software
#Foundation; either version 2 of the License, or (at your option) any later
#version.
#CCTDE is distributed in the  hope  that  it  will  be  useful, but WITHOUT ANY
#WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
#A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#You should have received a copy of the GNU General  Public  License along with
#CCTDE; if not, write to the Free Software  Foundation,  Inc., 59 Temple Place,
#Suite 330, Boston, MA  02111-1307  USA
#==============================================================================
#
# 1. Read logfile from stdin
# 2. Parse logfile and create next hash
#
# [tl1] [infos] => Creation, period, cl_num, tcp_con, packets, bytes
# [tl1] [Client 1] [infos] => ip, start, stop, con, prx, ptx, brx, btx, uptime
# [] [Client 1] [Server 1] [infos] => same as client infos + port
# [] [Client 1] [Server 2]
# [] [Client 1] ...
# [] ...
# [] [Client 2]
# ...
# [tl2]
#
#   

use GD::Graph::lines;

my %hash_infos;
my $intro;

my @graph_array_labels;
my @graph_array_val;
my @graph_array_val2;
my $init_clientX;
my @graph_setup;

sub build_hashes
{
    my $timeline = my $client = my $server = "";
    while (<>)
    {
	chomp;

	if (/^(CCTDE.*)/) { $intro = $1."<br>";	}
	if (/^(  o Processed.*)/) { $intro .= $1."<br>"; }
	if (/^(  o First packet.*)/) { $intro .= $1."<br>"; }

	if (/  o Time list created the (.*) and is (.*) secs long with (.*) clients./)
	{ 
	    $hash_infos {$1} {"infos"} {"period"} = $2;
	    $hash_infos {$1} {"infos"} {"cl_num"} = $3;
	    $timeline = $1;
	};

	if (! $timeline) { next; }
	if (/^    Tcp connections : (.*), Packets : (.*), Bytes: (.*)/)
	{
	    $hash_infos {$timeline} {"infos"} {"tcp_con"} = $1;
	    $hash_infos {$timeline} {"infos"} {"packets"} = $2;
	    $hash_infos {$timeline} {"infos"} {"bytes"} = $3;
	}
	if (/^    \# (.*) \[(.*) - (.*) \]/)
	{
	    $hash_infos {$timeline} {$1} {"infos"} {"start"} = $2;
	    $hash_infos {$timeline} {$1} {"infos"} {"stop"} = $3;
	    $client = $1;
	}

	if (! $client) { next; }
	if (/^      conn=(.*), packets rx=(.*) tx=(.*), bytes rx=(.*) tx=(.*), Uptime global=(.*)/)
	{
	    $hash_infos {$timeline} {$client} {"infos"} {"conn"} = $1;
	    $hash_infos {$timeline} {$client} {"infos"} {"prx"} = $2;
	    $hash_infos {$timeline} {$client} {"infos"} {"ptx"} = $3;
	    $hash_infos {$timeline} {$client} {"infos"} {"brx"} = $4;
	    $hash_infos {$timeline} {$client} {"infos"} {"btx"} = $5;
	    $hash_infos {$timeline} {$client} {"infos"} {"uptime"} = $6;
	}
	if (/^      \$ (.*) \[(.*) - (.*) \]/)
	{
	    $hash_infos {$timeline} {$client} {$1} {"infos"} {"start"} = $2;
	    $hash_infos {$timeline} {$client} {$1} {"infos"} {"stop"} = $3;
	    $server = $1;
	}

	if (! $server) { next; }
	if (/^        conn=(.*), packets rx=(.*) tx=(.*), bytes rx=(.*) tx=(.*), Uptime global=(.*)/)
	{
	    $hash_infos {$timeline} {$client} {$server} {"infos"} {"conn"} = $1;
	    $hash_infos {$timeline} {$client} {$server} {"infos"} {"prx"} = $2;
	    $hash_infos {$timeline} {$client} {$server} {"infos"} {"ptx"} = $3;
	    $hash_infos {$timeline} {$client} {$server} {"infos"} {"brx"} = $4;
	    $hash_infos {$timeline} {$client} {$server} {"infos"} {"btx"} = $5;
	    $hash_infos {$timeline} {$client} {$server} {"infos"} {"uptime"} = $6;
	}
    }
}

sub write_graph
{
    my $par_client = @_[0];
    my $par_what = @_[1];
    my $par_what2 = @_[2];
    my $flag = @_[3];
    
    my $filetype ="png";
    my $filename = "cctde-".$par_client."-".$par_what.($flag==0?"-":"_").$par_what2.".".$filetype;
    
    open (MG, '> '.$filename);
    binmode MG;
    print MG $cctde_clientX->$filetype;
    close MG;
    return ($filename);
}

sub init_graph
{
    my $par_client = @_[0];
    my $par_what = @_[1];
    my $par_what2 = @_[2];
    my $flag = @_[3];

    my $init_clientX = GD::Graph::lines->new(@graph_setup[0], @graph_setup[1]);
    $init_clientX->set( title => "CCTDE - ".$par_client." - ".$par_what.($flag==0?"-":"/").$par_what2,
			dclrs => ['green','red'],y_label => "Number per period",
			line_types => [1,1], line_width => 1,
			show_values => 1, x_labels_vertical => 1);
    $init_clientX->set_legend($par_what,$par_what2);

    if (@graph_setup[2] == 0) {
	$display_data = GD::Graph::Data->new([\@graph_array_label]); }
    if (@graph_setup[2] == 1) {
	$display_data = GD::Graph::Data->new([\@graph_array_label, \@graph_array_val]); }
    if (@graph_setup[2] == 2) {
	$display_data = GD::Graph::Data->new([\@graph_array_label, \@graph_array_val, \@graph_array_val2]); }

    $display_values = $display_data->copy;
    $init_clientX->set(show_values => $display_values);
    $cctde_clientX = $init_clientX->plot([\@graph_array_label, \@graph_array_val, \@graph_array_val2]) 
	or die $init_clientX->error;

    return (&write_graph($par_client,$par_what,$par_what2,$flag));
}

sub graph_clientX()
{
    my $par_time = @_[0];
    my $par_client = @_[1];
    my $par_what = @_[2];
    my $par_what2 = @_[3];
    my $flag = @_[4];

    my $val = $hash_infos {$par_time} {$par_client} {"infos"} {$par_what};
    my $val2 = $hash_infos {$par_time} {$par_client} {"infos"} {$par_what2};

    push @graph_array_label, $par_time;
    if ($flag == 0)
    {
	push @graph_array_val, $val;
	push @graph_array_val2, $val2;
    }
    if ($flag == 1)
    {
	$val = 1 if ($val == 0);
	$val2 = 1 if ($val2 == 0);
	push @graph_array_val, (($val / $val2) % 100);
	push @graph_array_val2, (($val2 / $val) * 100);
    }
}

sub parse_timeline_clientX()
{
    my @par = @_;
    my $par_client = @_[0];
    my $par_what1 = @_[1];
    my $par_what2 = @_[2];
    my $flag = @_[3];

    foreach $cur_time (sort keys %hash_infos)
    { 
	&graph_clientX($cur_time,$par_client,$par_what1,$par_what2,$flag); 
    }

    my $graph = &init_graph($par_client,$par_what1,$par_what2,$flag);

    undef @graph_array_val;
    undef @graph_array_val2;
    undef @graph_array_label;

    return $graph;
}

########
# MAIN #
########

print STDERR "Building hash from STDIN...\n";
build_hashes();

my $CLIENT = "192.168.19.14";

open (REPORT, '> '.$CLIENT.'.html');

print REPORT << "EOF";
<html>
<head><title>CCTDE - $CLIENT</title></head>
<body>
<font size=\"-1\">$intro</font>
<br>
EOF

print REPORT << "EOF";
<hr width=\"100%\"><br>
EOF

    @graph_setup[0] = 450; # width
    @graph_setup[1] = 350; # height
    @graph_setup[2] = 2;   # show_values (0,1,2)

$graph1 = &parse_timeline_clientX($CLIENT,"conn","",0);
$graph2 = &parse_timeline_clientX($CLIENT,"uptime","",0);
print REPORT << "EOF";
<center><img src=\"$graph1\">&nbsp;<img src=\"$graph2\"></center><br>
EOF

$graph1 = &parse_timeline_clientX($CLIENT,"prx","",0);
$graph2 = &parse_timeline_clientX($CLIENT,"ptx","",0);
print REPORT << "EOF";
<center><img src=\"$graph1\">&nbsp;<img src=\"$graph2\"></center><br>
EOF

$graph1 = &parse_timeline_clientX($CLIENT,"brx","",0);
$graph2 = &parse_timeline_clientX($CLIENT,"btx","",0);
print REPORT << "EOF";
<center><img src=\"$graph1\">&nbsp;<img src=\"$graph2\"></center><br>
EOF

print REPORT << "EOF";
<hr width=\"100%\">
EOF

    @graph_setup[0] = 450;
    @graph_setup[1] = 450;
    @graph_setup[2] = 0;

$graph1 = &parse_timeline_clientX($CLIENT,"brx","btx",0);
$graph2 = &parse_timeline_clientX($CLIENT,"brx","btx",1);
print REPORT << "EOF";
<center><img src=\"$graph1\">&nbsp;<img src=\"$graph2\"></center><br>
EOF

print REPORT << "EOF";
<hr width=\"100%\">
EOF

print REPORT << "EOF";
<font size=-1>
<p align=justify>
This file is part of CCTDE - Covert Channels and Tunnels Detection  Engine v0.1 (C) Simon Castro <scastro [at] entreelibre.com].<br>
CCTDE is free software; you can redistribute it  and/or  modify  it  under  the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.<br>
CCTDE is distributed in the  hope  that  it  will  be  useful, but WITHOUT  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.<br>
You should have received a copy of the GNU General  Public  License  along with CCTDE; if not, write to the Free Software  Foundation,  Inc., 59 Temple  Place, Suite 330, Boston, MA  02111-1307  USA
</p>
EOF

print REPORT << "EOF";
</body>
</html>
EOF
close REPORT;
