/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** Header files
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <signal.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/tcp.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include <sys/mman.h>

/*
** Snort related definitions
*/

#include "snort.h"
#include "spo_alert_unixsock.h"

/*
** CCTDE definitions
*/

#include "lists.h"
#include "tcp.h"
#include "storage.h"
#include "correlator.h"
#include "config.h"

#define NAME            "cctde"
#define LONGNAME        "Covert Channels and Tunnels Detection Engine"
#define VERSION_CODE    0x000100
#define DATE_CODE       0x060204
#define COPYRIGHT       "Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com"

#define CCTDE_VERSION(version)\
        (version >> 16) & 0xFF,\
        (version >> 8) & 0xFF,\
        version & 0xFF

#define MYVERSION NAME" - "LONGNAME" - v%d.%d.%d (%.2d/%.2d, 20%.2d)\n",\
        CCTDE_VERSION(VERSION_CODE),\
        CCTDE_VERSION(DATE_CODE)

/* for conf_upcon array options */
#define CF_MAX_UPCON_UPTIME 0
#define CF_MAX_UPCON_PRX 1
#define CF_MAX_UPCON_PTX 2
#define CF_MAX_UPCON_BRX 3
#define CF_MAX_UPCON_BTX 4

/* for other conf_xxx options */
#define CF_MAX_CON 0
#define CF_MAX_UPTIME 1
#define CF_MAX_PRX 2
#define CF_MAX_PTX 3
#define CF_MAX_BRX 4
#define CF_MAX_BTX 5

/* Global for conf_files */

#define MAX_LABEL 10
#define MAX_VAL 10

typedef struct s_conf_struct {
  char *label;
  int pos;
} t_conf_struct;

/* Located in conf_file.c */
extern t_conf_struct conf_upcon[];
extern t_conf_struct conf_max[];

typedef struct s_cctde_config
{
  char *conf_file;
  char *snort_label;
  char *pid_file_path;
  char *logfile;
  FILE *FS_logfile;

  char *cmd_file;
  char *bk_file;

  /* Displaying config. */
  int disp_tcp;
  int disp_cs;
  int disp_sm;
  int disp_tnc;
  int disp_res;
  int disp_cur;
  int disp_cli;
  int disp_sri;

  u_int32_t conf_upcon[5];

  u_int32_t conf_cl[6];
  u_int32_t conf_srv[6];
  u_int32_t conf_cl_dal[6];
  u_int32_t conf_srv_dal[6];
  u_int32_t conf_cl_pal[6];
  u_int32_t conf_srv_pal[6];

  u_int32_t conf_topX_cl[6];
  u_int32_t conf_topX_srv[6];

  int timeline_period;

  time_t not_before;
  time_t not_after;
} t_cctde_config;

/*
** Main struct
*/

typedef struct s_cctde_infos
{
  pid_t pid;
  time_t start_time;
  time_t start_time_pcap;
  time_t first_packet;
  time_t last_packet;

  t_cctde_config config;

  u_int32_t packets_number;
  u_int32_t tcp_conn_number;
  u_int32_t tcp_conn_alive_number;
  u_int32_t data_amount;
  u_int32_t desync_packets_number;

  /* The main storage area */
  t_list_main client_list;

  /* Use to track server infos without having to browse client_list */
  t_list_main server_list;

  /* Use to store TopX */
  t_list_main TOPX_SRV_con;
  t_list_main TOPX_SRV_uptime;
  t_list_main TOPX_SRV_prx;
  t_list_main TOPX_SRV_ptx;
  t_list_main TOPX_SRV_brx;
  t_list_main TOPX_SRV_btx;

  t_list_main TOPX_CL_con;
  t_list_main TOPX_CL_uptime;
  t_list_main TOPX_CL_prx;
  t_list_main TOPX_CL_ptx;
  t_list_main TOPX_CL_brx;
  t_list_main TOPX_CL_btx;

  /* Used to store rotated client_list */
  t_list_main time_list;
} t_cctde_infos;

extern t_cctde_infos G_cctde_infos;

extern u_int32_t OPT_timeline;

/*
** Functions definitions
*/

/* cctde.c */
void sig_term (int sig);
void sig_hup(int sig);
void sig_usr1(int sig);
void init_cctde_infos();
int manage_packet(Alertpkt alert,Packet *p);
int loop_on_unixsock(void);
int main(int ac, char **av);

/* cctde_utils.c */
int display_infos(FILE *fstream, char *fmt, ...);
int create_pid_file();
FILE *get_stream_from_file(char *filename,int open_type);
void version(FILE *fstream);
void synopsis(void);
void display_cctde_infos();
time_t current_time();
time_t running_time(time_t last, int flag);
char *ip_uint_to_str(u_int32_t ip);
u_int32_t get_uint_ip_from_str(char *str_ip);
char *get_str_from_time_t(time_t time);
char *get_str2_from_time_t(time_t time);

/* snort_src/util.c */
void ts_print(register const struct timeval *tvp, char *timebuf);
int gmt2local(time_t t);

/* conf_file.c */
int  parse_conf_file(int (browse_conf)(), char *filename);
int search_config_function(char *function,char *params);

/* execute_cmd_file.c */
int exec_search(char *function, char *params, int flag);
int exec_display(char *function, char *params, int flag);
int build_topX(char *function, char *params, int flag);
int execute_cmd_file(char *function,char *params);

/* backup.c */
int backup_cctde_infos(char *function, char *params, int flag);
int read_stats_from(char *function, char *params, int flag);
int backup_timeline(t_time_storage *time_storage, FILE *fs_file);
