/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define CONSIDER_TIMELINE() (G_cctde_infos.config.timeline_period == 0 ? 0 : 1)
#define ROTATION_PERIOD() (G_cctde_infos.config.timeline_period)

#define STORAGE_SRV_MAGIC_S 'A'
#define STORAGE_SRV_MAGIC_E 'B'
#define STORAGE_CL_MAGIC_S 'C'
#define STORAGE_CL_MAGIC_E 'D'
#define STORAGE_TP_MAGIC_S 'E'
#define STORAGE_TP_MAGIC_E 'F'

/*
** The s_storage struct stores informations for clients
** and servers.
*/

typedef struct s_alert {
  time_t last_alert;
  u_int32_t alert_value;
} t_alert;

#define ALERT_STATS_CON 0
#define ALERT_STATS_UPTIME 1
#define ALERT_STATS_PRX 2
#define ALERT_STATS_PTX 3
#define ALERT_STATS_BRX 4
#define ALERT_STATS_BTX 5

typedef struct s_storage {
  u_char magic_s;

  u_int32_t ip;
  u_int16_t port;
  u_int32_t tcp_nb;
  u_int32_t packet_rx;
  u_int32_t packet_tx;
  u_int32_t bytes_rx;
  u_int32_t bytes_tx;

  t_alert alert_stats[6];

  struct timeval time_creation; /* Element creation */
  struct timeval time_first_access; /* Last connection creation */
  struct timeval time_last_access; /* Last packet sent/received */
  u_int32_t com_uptime;

  /* Only used on the first level client_list */
  t_list_main second_list;
  t_list_elem *sec_list_last_acceded;
  u_int32_t sec_list_qt;

  /* Only used on the second level client_list to
     store a ptr to the related server_list elem */
  t_list_elem *server_infos;

  u_char magic_e;
} t_storage;

typedef struct s_time_storage {
  u_char magic_s;

  time_t time_creation;
  u_int32_t period;

  u_int32_t packets_number;
  u_int32_t tcp_conn_number;
  u_int32_t data_amount;

  u_int32_t client_number;

  t_list_main client_list;

  u_char magic_e;
} t_time_storage;

/* storage.c */
t_storage build_storage_view(t_workflow workflow,Alertpkt alert,Packet *p,int stype);
int match_timeperiod(t_time_storage *stored, struct timeval *timeval);
int match_ip(t_storage *stored_view, t_storage *tmp_view);
int match_ipport(t_storage *stored_view, t_storage *tmp_view);
int update_stored_view(t_storage *stored_view, t_storage *tmp_view);
t_list_elem *store_flow(t_workflow workflow,Alertpkt alert,Packet *p);
u_int32_t get_start_period();
t_list_elem *consider_timeline(t_workflow workflow,Alertpkt alert,Packet *p);

/* storage_display.c */
void display_timeline(t_time_storage *time_storage);
void display_clients(t_storage *client_infos);
void display_2nd_servers(t_storage *stored_infos);
void display_clients_and_servers(t_storage *client_infos);
void display_servers(t_storage *server_infos);
int display_top_servers();
int display_top_clients();

/* storage_topX.c */
int wgrteq_con(t_storage *stored_view, t_storage *tmp_view);
int wgrteq_uptime(t_storage *stored_view, t_storage *tmp_view);
int wgrteq_prx(t_storage *stored_view, t_storage *tmp_view);
int wgrteq_ptx(t_storage *stored_view, t_storage *tmp_view);
int wgrteq_brx(t_storage *stored_view, t_storage *tmp_view);
int wgrteq_btx(t_storage *stored_view, t_storage *tmp_view);

int store_top_X(t_list_elem *stored, t_list_main *lm, int (*match_fn)(), int (*top_fn)(), int top_size);
