/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** Connection tracking is taken of or inspired from tcpstatflow v1.1
*/

/* The flow struct used to get the hash */

typedef struct s_flow {
  u_int32_t src;
  u_int32_t dst;
  u_int16_t sport;
  u_int16_t dport;
} t_flow;

/* States of the flow */

#define FLOW_STATE_UNKNOWN 0
#define FLOW_STATE_ON_CONNECT 1
#define FLOW_STATE_CONNECTED 2
#define FLOW_STATE_CLOSING 3
#define FLOW_STATE_TO_CLOSE 4

/* Struct. element for connected flows */

typedef struct s_con_track {
  struct s_con_track *next;
  struct s_con_track *prev;
  t_flow flow_infos;
  u_char state;
  u_int32_t isn_c;
  u_int32_t isn_s;

  u_int32_t packet_rx;
  u_int32_t packet_tx;
  u_int32_t bytes_rx;
  u_int32_t bytes_tx;
  struct timeval time_first_access; 
  struct timeval time_last_access; 

  /* pointer to this client/server informations */
  t_list_elem *infos;
} t_con_track;

/* Working struct. */

typedef struct s_workflow {
  t_flow flow_infos;
  u_int32_t flow_hash;
  t_con_track *ptr;
} t_workflow;

/* Macros */

#define CON_HASH_SIZE 133981
extern t_con_track *G_con_track[CON_HASH_SIZE];
#define HASH_FLOW(flow) ( ( \
( (flow->sport & 0xffff) ^ (flow->dport & 0xffff) ) | \
( ( (flow->src & 0xffff) ^ (flow->dst & 0xffff) ) << 16 ) \
) % CON_HASH_SIZE)

#define FLOW_WAY(stored,packet) ((stored->flow_infos.src == ntohl(packet->iph->ip_src.s_addr)) &&\
(stored->flow_infos.sport == ntohs(packet->tcph->th_sport)) ? 0 : 1)

/* Prototypages */

/* tcp.c */
void init_con_track();
u_int32_t get_tcp_flow_infos(t_flow *flow_infos, Packet *p);
t_flow get_tcp_rev_flow_infos(t_flow flow_infos);
t_con_track *add_con_track_flow(t_workflow *workflow);
void display_flow(FILE *stream,t_flow flow);
void display_con_track_flows();
t_con_track *find_con_track_flow(t_workflow *workflow);
int remove_con_track_flow(t_workflow workflow);
int display_tcp_connection(t_workflow workflow, int type);
int manage_tcp_connection(Alertpkt alert,Packet *p);
