/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

int backup_timeline3(t_storage *server_storage, FILE *fs_file)
{
  if (fwrite(server_storage,sizeof(t_storage),1,fs_file) != 1)
    return (1);

  // Return 0 to browse the whole list
  return (0);
}

int backup_timeline2(t_storage *client_storage, FILE *fs_file)
{
  static t_storage tmp;
  memcpy(&tmp,client_storage,sizeof(t_storage));
  memset(&(tmp.second_list),0,sizeof(t_list_main));
  if (fwrite(&tmp,sizeof(t_storage),1,fs_file) != 1)
    return (1);

  if (reverse_look_for_elem(&(client_storage->second_list),fs_file,backup_timeline3) != NULL)
    return (1);

  // Return 0 to browse the whole list
  return (0);
}

int backup_timeline(t_time_storage *time_storage, FILE *fs_file)
{
  static t_time_storage tmp;
  memcpy(&tmp,time_storage,sizeof(t_time_storage));
  memset(&(tmp.client_list),0,sizeof(t_list_main));
  if (fwrite(&tmp,sizeof(t_time_storage),1,fs_file) != 1)
    return (1);

  if (time_storage->client_number != 0)
    if (reverse_look_for_elem(&(time_storage->client_list),fs_file,backup_timeline2) != NULL)
      return (1);

  // Return 0 to browse the whole list
  return (0);
}

int backup_main(FILE *fs_file)
{
  static t_time_storage tmp;

  memset(&tmp,0,sizeof(t_time_storage));
  tmp.magic_s = STORAGE_TP_MAGIC_S;
  tmp.magic_e = STORAGE_TP_MAGIC_E;
  tmp.packets_number = G_cctde_infos.packets_number;
  tmp.tcp_conn_number = G_cctde_infos.tcp_conn_number;
  tmp.data_amount = G_cctde_infos.data_amount;
  tmp.client_number = G_cctde_infos.client_list.elem_nb;
  tmp.time_creation = get_start_period();
  tmp.period = G_cctde_infos.last_packet - tmp.time_creation;

  if (fwrite(&tmp,sizeof(t_time_storage),1,fs_file) != 1)
    return (1);

  if (G_cctde_infos.client_list.elem_nb > 0)
    if (reverse_look_for_elem(&(G_cctde_infos.client_list),fs_file,backup_timeline2) != NULL)
      return (1);

  // Return 0 to browse the whole list
  return (0);
}

int backup_cctde_infos(char *function, char *params, int flag)
{
  FILE *info_file;

  if (!(info_file = get_stream_from_file(params,1)))
    {
      display_infos(stderr,"Cannot open %s to backup infos!\n",params);
      return (0);
    }

  // We backup the whole timeline struct elem after elem, list after list, etc...
  // I had two choices :
  // Create an array for clients and servers => x malloc and 3 fwrite for 1 time period
  // or parse the whole lists using fwrite each time I have an element to store...
  // I choosed the second way :|
  if (CONSIDER_TIMELINE() != 0)
    {
      // Error is reversed : if we have an element, it means we couldn't browse the
      // whole list so there is an error somewhere ...
      if (reverse_look_for_elem(&(G_cctde_infos.time_list),info_file,backup_timeline) != NULL)
	{
	  display_infos(stderr,"Problem while backuping timeline!\n");
	  fclose(info_file);
	  return (0);
	}
    }

  // We backup the current cctde infos in a timeline format only if we have
  // recorded clients...
  if (G_cctde_infos.client_list.elem_nb > 0)
    if (backup_main(info_file) == 1)
      {
	display_infos(stderr,"Problem while backuping cctde infos!\n");
	fclose(info_file);
	return (0);
      }

  fclose(info_file);
  return (1);
}

int read_stats_from(char *function, char *params, int flag)
{
  FILE *fs_file=NULL;
  int fd_file,pos=0,rd=0;
  void *pos_mmap;
  char *ptr_mmap;
  struct stat pt;
  t_time_storage time_storage;
  t_storage tmp_storage, *tmp_cast2;
  t_time_storage *tmp_cast1;
  t_list_elem *time_list_elem=NULL,*client_list_elem=NULL;

  if (!params || strlen(params) < 1)
    return (0);

  if (flag == 0)
    {
      if (!(fs_file = get_stream_from_file(params,0)))
	return (0);
      if ((fd_file = fileno(fs_file)) == -1)
	return (0);
      if (fstat(fd_file,&pt) == -1 || pt.st_size < 4)
	return (0);
      if ((pos_mmap = mmap(0,pt.st_size,PROT_READ,MAP_PRIVATE,fd_file,0)) == MAP_FAILED)
	return (0);

      ptr_mmap = (char *) pos_mmap;

      while (rd < pt.st_size)
	{
	  if (ptr_mmap[pos] == STORAGE_TP_MAGIC_S)
	    {
	      memset(&time_storage,0,sizeof(t_time_storage));
	      memcpy(&time_storage,pos_mmap+pos,sizeof(t_time_storage));
	      pos += sizeof(t_time_storage);
	      rd += sizeof(t_time_storage);
	      
	      if ((time_list_elem = add_to_list(&(G_cctde_infos.time_list),
						&time_storage,sizeof(t_time_storage),1)) == NULL)
		return (0);
	    }
	  else
	    {
	      if (ptr_mmap[pos] == STORAGE_CL_MAGIC_S || ptr_mmap[pos] == STORAGE_SRV_MAGIC_S)
		{
		  memset(&tmp_storage,0,sizeof(t_storage));
		  memcpy(&tmp_storage,pos_mmap+pos,sizeof(t_storage));
		  if (ptr_mmap[pos] == STORAGE_CL_MAGIC_S)
		    {
		      if (!time_list_elem)
			return (0);
		      tmp_cast1 = (t_time_storage *) time_list_elem->data;
		      if ((client_list_elem = add_to_list(&(tmp_cast1->client_list),&tmp_storage,sizeof(t_storage),1)) == NULL)
			return(0);
		    }
		  else
		    {
		      if (!client_list_elem)
			return (0);
		      tmp_cast2 = (t_storage *) client_list_elem->data;
		      if ((add_to_list(&(tmp_cast2->second_list),&tmp_storage,sizeof(t_storage),1)) == NULL)
		      return(0);
		    }
		  pos += sizeof(t_storage);
		  rd += sizeof(t_storage);
		}
	      else
		{
		  display_infos(stderr,"Cannot find magic in bk file... walking ...\n");
		  pos += 1;
		  rd += 1;
		}
	    }
	}
      if (munmap(0,pt.st_size) == -1)
	return (0);
      fclose(fs_file);
      return (1);
    }
  return (1);
}
