/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

int G_usd; /* Unix socket descriptor used by the Snort process */
t_cctde_infos G_cctde_infos; /* Statistics for the cctde process */
u_int32_t G_last_refresh;

u_int32_t OPT_refresh, OPT_starttime;
u_int32_t OPT_NB, OPT_NA;
char *OPT_sockfile,*OPT_logfile;
char *OPT_confile,*OPT_pid_file_path;

int G_signal_received;

/*
** Init. and signals
*/

void sig_term(int sig)
{
  display_cctde_infos();
  display_infos (stderr,"SigTERM ... Exiting!\n");
  close (G_usd);
  unlink (OPT_sockfile);
  if (G_cctde_infos.config.FS_logfile)
    fclose(G_cctde_infos.config.FS_logfile);
  if (G_cctde_infos.config.bk_file)
    backup_cctde_infos("TermBackup",G_cctde_infos.config.bk_file,0);

  exit (1);
}

void sig_usr2(int sig)
{
  if (parse_conf_file(execute_cmd_file,G_cctde_infos.config.cmd_file) == -1)
    display_infos(stderr,"Cannot execute commands in %s...\n",G_cctde_infos.config.cmd_file);
  G_signal_received = 1;
}

void sig_usr1(int sig)
{
  display_cctde_infos();
  G_signal_received = 1;
}

void sig_hup(int sig)
{
  if (parse_conf_file(search_config_function,G_cctde_infos.config.conf_file) == -1)
    display_infos(stderr,"Errors in %s... Exit.\n",G_cctde_infos.config.conf_file);
  G_signal_received = 1;
}

void init_cctde_infos()
{
  time_t cur_time;

  time(&cur_time);
  bzero(&G_cctde_infos,sizeof(t_cctde_infos));
  G_cctde_infos.start_time = cur_time;
  G_cctde_infos.pid = getpid();
  init_main_list(&G_cctde_infos.client_list);
  init_main_list(&G_cctde_infos.time_list);
  G_last_refresh = G_cctde_infos.start_time;
  OPT_refresh = 0;
  OPT_starttime = 0;
  OPT_sockfile=NULL;
  OPT_confile=NULL;
  OPT_logfile = NULL;
  OPT_pid_file_path = NULL;
  G_signal_received = 0;
}

/*
** Managing packets : currently only tcp...
*/

int manage_packet(Alertpkt alert,Packet *p)
{
  int ret;

  /* First check we have a tcp packet header ... */
  if (!p->tcph)
    return (0);

  /* 
  ** We first check that the Snort alert message is the one we
  ** want for the cctde monitoring.
  ** Planned : Manage more than one alertmsg to take in consideration
  ** the "colored" lists functionalities.
  */

  if ((strlen(alert.alertmsg) != strlen(G_cctde_infos.config.snort_label)) ||
      strncmp(alert.alertmsg,G_cctde_infos.config.snort_label,
	      strlen(G_cctde_infos.config.snort_label)) != 0)
    {
      if (G_cctde_infos.config.disp_sm == 1)
	display_infos(stdout,"Snort told: '%s'\n",alert.alertmsg);
      return (0);
    }

  /*
  ** Only manage tcp because we want http based detection.
  */

  if (!(alert.val & NOPACKET_STRUCT))
    if (p->iph && p->tcph)
      {
	ret = manage_tcp_connection(alert,p);
	return (ret);
      }
  return (0);
}

/*
** Looping on Unix Socket
*/

int loop_on_unixsock(void)
{
  struct sockaddr_un snortaddr;
  struct sockaddr_un bogus;
  Alertpkt alert;
  Packet *p=NULL;
  int recv;
  socklen_t len = sizeof (struct sockaddr_un);
  int ret,go_recv;
  int refresh_runtime;

  fd_set fdset_read;
  struct timeval fd_tv;
  
  if ((G_usd = socket (AF_UNIX, SOCK_DGRAM, 0)) < 0)
    {
      display_infos(stderr,"Cannot open socket.\n");
      return (-1);
    }

  bzero (&snortaddr, sizeof (snortaddr));
  snortaddr.sun_family = AF_UNIX;
  strcpy (snortaddr.sun_path, OPT_sockfile);

  if (bind (G_usd, (struct sockaddr *) &snortaddr, sizeof (snortaddr)) < 0)
    {
      display_infos(stderr,"Cannot bind socket.\n");
      return (-1);
    }

  signal(SIGINT, sig_term);
  signal(SIGTERM, sig_term);
  signal(SIGHUP, sig_hup);
  signal(SIGUSR1, sig_usr1);
  signal(SIGUSR2, sig_usr2);

  while (1)
    {
      FD_ZERO(&fdset_read);
      FD_SET(G_usd,&fdset_read);
      fd_tv.tv_sec = 1;
      fd_tv.tv_usec = 0;
      go_recv = recv = 0;
      
      if ((select(G_usd+1,&fdset_read,NULL,NULL,&fd_tv)) < 0)
	{
	  if (G_signal_received > 0)
	    G_signal_received = 0;
	  else
	    {
	      display_infos(stderr,"Select error.\n");
	      return (-1);
	    }
	}
      else
	if (FD_ISSET(G_usd,&fdset_read))
	  go_recv = 1;

      if (go_recv == 1 && (recv = recvfrom (G_usd, (void *) &alert, sizeof (alert),
					    0, (struct sockaddr *) &bogus, &len)) > 0)
	{
	  if (!(alert.val & NOPACKET_STRUCT))
	    {
	      if ((p = calloc (1, sizeof (Packet))) == NULL)
		{
		  display_infos(stderr,"Calloc error.\n");
		  return (-1);
		}
	      
	      p->pkt = alert.pkt;
	      p->pkth = &alert.pkth;
	      if (alert.dlthdr)
		p->eh = (EtherHdr *) (alert.pkt + alert.dlthdr);
	      
	      if (alert.nethdr)
		{
		  p->iph = (IPHdr *) (alert.pkt + alert.nethdr);
		  if (alert.transhdr && p->iph->ip_proto == IPPROTO_TCP)
		    {
		      /* We only keep TCP here as we are doing http detection */
		      p->tcph = (TCPHdr *) (alert.pkt + alert.transhdr);
		    }
		}
	      
	      if (alert.data)
		p->data = alert.pkt + alert.data;
	      
	      if ((ret = manage_packet(alert,p)) < 1)
		{
		  if (ret == -1) // There was a fatal error, returns now !
		    {
		      display_infos(stderr,"Manage_packet error.\n");
		      return (-1);
		    }
		}
	    }
	}
      else
	if (recv == -1)
	  {
	    display_infos(stderr,"Recv error.\n");
	    return (-1);
	  }

      /* Select returned or got a packet */

      /* Wanna display infos with the refresh rate ? */

      if (OPT_refresh != 0)
	{
	  if (G_cctde_infos.start_time_pcap != 0)
	    {
	      if (G_last_refresh == G_cctde_infos.last_packet)
		refresh_runtime = 0;
	      else
		refresh_runtime = running_time(G_cctde_infos.last_packet,1);
	      
	      if (refresh_runtime != 0 && refresh_runtime > OPT_refresh)
		{
		  display_cctde_infos();
		  G_last_refresh = G_cctde_infos.last_packet;
		}
	    }
	  else
	    {
	      if (G_last_refresh == current_time())
		refresh_runtime = 0;
	      else
		refresh_runtime = running_time(G_cctde_infos.start_time,0);

	      // Avoid displaying infos for each packet coming the same sec
	      if (refresh_runtime != 0 && (refresh_runtime % OPT_refresh == 0))
		{
		  display_cctde_infos();
		  G_last_refresh += OPT_refresh;
		}
	    }	 	  
	}
      
      /* Begin correlation ... */
      correlate_and_alert();
    }
  return (0);
}


/*
** I should do checks for each values... maybe next time ?
*/ 

void check_optargs()
{
  if (OPT_starttime != 0)
    G_cctde_infos.start_time_pcap = OPT_starttime;

  if (OPT_confile)
    {
      if (!(G_cctde_infos.config.conf_file = (char *) malloc(strlen(OPT_confile)+1)))
	{
	  display_infos(stderr,"check_optargs ... Malloc error\n");
	  exit (-1);
	}
      memset(G_cctde_infos.config.conf_file,0,strlen(OPT_confile)+1);
      memcpy(G_cctde_infos.config.conf_file,OPT_confile,strlen(OPT_confile));
    }
  else
    G_cctde_infos.config.conf_file = DEFAULT_CONF_FILE;

  if (parse_conf_file(search_config_function,G_cctde_infos.config.conf_file) == -1)
    {
      display_infos(stderr,"Errors in %s... Exit.\n",G_cctde_infos.config.conf_file);
      exit (-1);
    }

  if (OPT_pid_file_path)
    {
      if (!(G_cctde_infos.config.pid_file_path = (char *) malloc(strlen(OPT_pid_file_path)+1)))
	{
	  display_infos(stderr,"check_optargs ... Malloc error\n");
	  exit (-1);
	}
      memset(G_cctde_infos.config.pid_file_path,0,strlen(OPT_pid_file_path)+1);
      memcpy(G_cctde_infos.config.pid_file_path,OPT_pid_file_path,strlen(OPT_pid_file_path));
      if (create_pid_file() == -1)
	{
	  display_infos(stderr,"Cannot create pid_file.\nExiting...\n");
	  exit (-1);
	}
    }

  if (OPT_logfile)
    {
      if (!(G_cctde_infos.config.logfile = (char *) malloc(strlen(OPT_logfile)+1)))
	{
	  display_infos(stderr,"check_optargs ... Malloc error\n");
	  exit (-1);
	}
      memset(G_cctde_infos.config.logfile,0,strlen(OPT_logfile)+1);
      memcpy(G_cctde_infos.config.logfile,OPT_logfile,strlen(OPT_logfile));
      if ((G_cctde_infos.config.FS_logfile = 
	   get_stream_from_file(OPT_logfile,1)) == NULL)
	{
	  display_infos(stderr,"Cannot open %s logfile\n",OPT_logfile);
	  exit (-1);
	}
    }

  if (OPT_NB)
    G_cctde_infos.config.not_before = OPT_NB;

  if (OPT_NA)
    G_cctde_infos.config.not_after = OPT_NA;

  if (!G_cctde_infos.config.snort_label)
    G_cctde_infos.config.snort_label = SNORT_LABEL;
}

/*
** Main : Init. then loop on receiving datas from unix socket...
*/

int main(int ac, char **av)
{
  int arg_nb;

  init_cctde_infos();

  while ((arg_nb = getopt(ac,av, "S:f:s:R:p:l:B:A:Vh")) != EOF) {
    switch(arg_nb) {
    case 'S':
      OPT_sockfile = optarg;
      break;
    case 'V':
      version(stderr);
      exit(0);
      break;
    case 'h':
      synopsis();
      exit(0);
      break;
    case 'f':
      OPT_confile = optarg;
      break;
    case 's':
      OPT_starttime = (time_t) atoi(optarg);
      break;
    case 'R':
      OPT_refresh = atoi(optarg);
      break;
    case 'p':
      OPT_pid_file_path = optarg;
      break;
    case 'l':
      OPT_logfile = optarg;
      break;
    case 'B':
      OPT_NB = atoi(optarg);
      break;
    case 'A':
      OPT_NA = atoi(optarg);
      break;
    default:
      break;
    }
  }

  if (!OPT_sockfile)
    {
      display_infos(stderr,"Error : -S Snort_usock is mandatory.\n");
      synopsis();
    }

  init_con_track();
  check_optargs();

  if (loop_on_unixsock() == -1)
    display_infos(stdout,"FATAL ERROR !!!\n");
  close (G_usd);
  unlink (OPT_sockfile);
  return 0;
}
