/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

int display_infos(FILE *fstream, char *fmt, ...)
{
  va_list vlist;

  va_start(vlist,fmt);

  if (G_cctde_infos.config.FS_logfile)
    {
      vfprintf(G_cctde_infos.config.FS_logfile,fmt,vlist);
      fflush(G_cctde_infos.config.FS_logfile);
    }
  else
    {
      vfprintf(fstream,fmt,vlist);
      fflush(fstream);
    }

  va_end(vlist);
  return (-1);
}

void version(FILE *fstream)
{
  display_infos(fstream,"\n");
  display_infos(fstream,MYVERSION);
}

void synopsis(void)
{
  version(stderr);
  display_infos(stderr,"%s\n",COPYRIGHT);
  display_infos(stderr,
	  "\nSynopsis:\n---------\n"
          "\n"
	  "  %s -S snort_usock [-f config_file] [-p pidfilepath] [-l logfile]"
	  "  [-s start_time] [-R refresh] [-B not_before] [-A not_after]\n"
	  "  %s -V\n"
	  "  %s -h\n\n",
	  NAME,NAME,NAME);
  display_infos(stderr,
          "Parameters:\n-----------\n"
          "\n"
	  "  %s\t-S snort_usock\tPath to the Snort unix socket (ex: logs/snort_alert).\n"
	  "\n"
	  "  %s\t-f config_file\tUse the configuration file (if NS, use DEFAULT_CONF_FILE).\n"
	  "  %s\t-p pidfilepath\tLog pid to pidfilepath/cctde.pid.\n"
	  "  %s\t-l logfile\tLog to logfile.\n"
	  "  %s\t-s start_time\tDo as if start_time was the current time (ex: 1074194932).\n"
	  "  %s\t-R refresh\tVerbose refresh period delay (in secs).\n"
	  "  %s\t-B not_before\tDon't take packet if tv_sec < not_before (ex: 1074194932).\n"
	  "  %s\t-A not_after\tDon't take packet if tv_sec > not_after (ex: 1074194932).\n"
	  "\n"
          "  %s\t-V\t\tShow version.\n"
          "  %s\t-h\t\tShow this help.\n",
	  NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME);
  display_infos(stderr,
          "\nSignals:\n--------\n"
          "\n"
	  "  Sighup\tReload configuration file.\n"
	  "  Sigusr1\tCall display_cctde_infos.\n"
	  "  Sigusr2\tExecute functions from A_COMMAND_FILE.\n"
	  );
  exit(0);
}

#define PID_FILE_MLENGTH 256
int create_pid_file()
{
  FILE *pid_file;
  struct stat pt;
  static char filename[PID_FILE_MLENGTH+1];
  int path_lg,path_fn;

  stat(G_cctde_infos.config.pid_file_path,&pt);
  if(!S_ISDIR(pt.st_mode) || access(G_cctde_infos.config.pid_file_path, W_OK) == -1)
    return (-1);

  path_lg = strlen(G_cctde_infos.config.pid_file_path);
  path_fn = strlen(PIDFILENAME);
  if (path_lg+path_fn>PID_FILE_MLENGTH)
    return (-1);
  memset(filename,0,PID_FILE_MLENGTH+1);
  strncpy(filename,G_cctde_infos.config.pid_file_path,path_lg);

  if (filename[path_lg-1] == '/')
    strncpy(filename+path_lg,PIDFILENAME,path_fn);
  else
    {
      filename[path_lg] = '/';
      strncpy(filename+path_lg+1,PIDFILENAME,path_fn);
    }

  if ((pid_file = fopen(filename,"w")))
    {
      display_infos(pid_file,"%d\n",G_cctde_infos.pid);
      fclose(pid_file);
    }
  else
    return (-1);

  return (1);
}

FILE *get_stream_from_file(char *filename,int open_type)
{
  FILE *fs_file;

  if (open_type == 0 && (fs_file = fopen(filename,"r")) != NULL)
    return (fs_file);
  if (open_type == 1 && (fs_file = fopen(filename,"w")) != NULL)
    return (fs_file);
  return (NULL);
}

/*
** Debug, basic infos, etc.
*/

void display_cctde_infos()
{
  // Main infos 
  if (G_cctde_infos.config.disp_res != 0)
    {
      display_infos(stdout,"\nCCTDE [%d] started the %s",
	       G_cctde_infos.pid,get_str_from_time_t(G_cctde_infos.start_time));
      if (G_cctde_infos.start_time_pcap != 0)
	display_infos(stdout," (Pcap start : %s)\n",get_str_from_time_t(G_cctde_infos.start_time_pcap));
      else
	display_infos(stdout,"\n");
      
      if (G_cctde_infos.start_time_pcap != 0)
	display_infos(stdout,"  o Processed %d connections and %d bytes (%d packets) in %d seconds (%d desync packets) (Pcap delay : %d).\n",
		      G_cctde_infos.tcp_conn_number,G_cctde_infos.data_amount,
		      G_cctde_infos.packets_number,(int) running_time(G_cctde_infos.start_time,0),
		      G_cctde_infos.desync_packets_number,
		      (int) running_time(G_cctde_infos.last_packet,1));
      else
	display_infos(stdout,"  o Processed %d connections and %d bytes (%d packets) in %d seconds (%d desync packets).\n",
		      G_cctde_infos.tcp_conn_number,G_cctde_infos.data_amount,
		      G_cctde_infos.packets_number,(int) running_time(G_cctde_infos.start_time,0),
		      G_cctde_infos.desync_packets_number);
      
      display_infos(stdout,"  o First packet : %s",get_str_from_time_t(G_cctde_infos.first_packet));
      display_infos(stdout," / Last packet : %s\n",get_str_from_time_t(G_cctde_infos.last_packet));
      if (G_cctde_infos.config.not_before != 0)
	display_infos(stdout,"  o Not accepting new connections before : %s\n",
		      get_str_from_time_t(G_cctde_infos.config.not_before));
      if (G_cctde_infos.config.not_after != 0)
	display_infos(stdout,"  o Not accepting new connections after : %s\n",
		      get_str_from_time_t(G_cctde_infos.config.not_after));
    }
	   
  // Current connections
  if (G_cctde_infos.config.disp_cur != 0)
    {
      display_infos(stdout,"  o %d connection(s) UP.\n",G_cctde_infos.tcp_conn_alive_number);
      display_con_track_flows();
    }

  // Clients infos
  if (G_cctde_infos.config.disp_cli == 1 || G_cctde_infos.config.disp_cli == 2)
    {
      display_infos(stdout,"  o %d recorded client(s).\n",G_cctde_infos.client_list.elem_nb);
      if (G_cctde_infos.config.disp_cli == 1)
	browse_existing_elems(&(G_cctde_infos.client_list),display_clients);
      else
	browse_existing_elems(&(G_cctde_infos.client_list),display_clients_and_servers);
    }

  // Server infos
  if (G_cctde_infos.config.disp_sri != 0)
    {
      display_infos(stdout,"  o %d recorded server(s).\n",G_cctde_infos.server_list.elem_nb);
      browse_existing_elems(&(G_cctde_infos.server_list),display_servers);
    }

  // TOP INFOS
  display_top_clients();
  display_top_servers();

  // Timeline infos
  browse_existing_elems(&(G_cctde_infos.time_list),display_timeline);
}

time_t current_time()
{
  time_t cur_time;

  return (time(&cur_time));
}

time_t running_time(time_t last,int flag)
{
  time_t cur_time;

  time(&cur_time);

  if (flag < 1 || flag > 1) // Get from current time
    return (last == 0 ? 0 : cur_time - last);

  if (flag == 1)
    return (last == 0 ? 0 : last - G_cctde_infos.start_time_pcap);

  return (last == 0 ? 0 : cur_time - last);
}

/*
** Useful functions
*/

char *ip_uint_to_str(u_int32_t ip)
{
  static char address[16];

  memset(address,0,16);
  snprintf(address,16,"%d.%d.%d.%d",
          (u_int8_t) ((ip & 0xff000000) >> 24),
          (u_int8_t) ((ip & 0x00ff0000) >> 16),
          (u_int8_t) ((ip & 0x0000ff00) >> 8),
          (u_int8_t)  (ip & 0x000000ff));

  return (address);
}

u_int32_t get_uint_ip_from_str(char *str_ip)
{
  struct hostent *he;
  struct sockaddr_in sin;

  if (!(he = (struct hostent *) gethostbyname(str_ip)))
    return (0);

  memcpy(&sin.sin_addr,he->h_addr,sizeof(sin.sin_addr));
  return ((u_int32_t) htonl(sin.sin_addr.s_addr));
}

char *get_str_from_time_t(time_t time)
{
  static char str[24];
  struct tm *tm_struct;

  memset(str,0,24);
  if ((tm_struct = localtime((time_t *)&time)) == NULL)
    return ("time_error");

  if (strftime(str,23,"%d/%m/%Y %T",tm_struct) == 0)
    return ("time_error");

  return (str);
}

char *get_str2_from_time_t(time_t time)
{
  static char str[12];
  struct tm *tm_struct;

  memset(str,0,12);
  if ((tm_struct = localtime((time_t *)&time)) == NULL)
    return ("time_error");

  if (strftime(str,11,"%s",tm_struct) == 0)
    return ("time_error");
  return (str);
}
