/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"


/*
** Detecting suspect flows on stored client infos
*/

int basic_client_check_val(u_int32_t stored_val,u_int32_t check_val)
{
  if (stored_val > check_val)
    return (1);
  else
    if (stored_val == check_val)
      return (0);
  return (-1);
}

int generic_basic_stats(t_storage *stored_view,u_int32_t check_val, u_int32_t max_val,
			u_int32_t alert_type, u_int32_t reminder_del, u_int32_t reminder_per)
{
  int ret=-1;
  int tmp;
  u_int32_t aval=0,atime=0;

  if ((tmp = basic_client_check_val(check_val,max_val)) >= 0)
    {
      aval = stored_view->alert_stats[alert_type].alert_value;
      atime = stored_view->alert_stats[alert_type].last_alert;
      if (aval == 0 || atime == 0) // First alert
	{
	  stored_view->alert_stats[alert_type].alert_value = check_val;
	  stored_view->alert_stats[alert_type].last_alert = current_time();
	  return (0);
	}

      if (reminder_del != 0 && (current_time() >= (atime + reminder_del)))
	{
	  stored_view->alert_stats[alert_type].last_alert = current_time();
	  ret = 1;
	}

      if (reminder_per != 0 && 
	  ((((check_val - aval) * 100) / aval) >= reminder_per))
	{
	  stored_view->alert_stats[alert_type].alert_value = check_val;
	  ret = 2;
	}

    }
  return (ret);
}

int basic_client_stats(t_storage *stored_view, t_storage *null_ptr)
{
  int ret;
  /* Maximum threshold, Delay before remind, percentage before remind... */
  u_int32_t max,del,per; 

  max = G_cctde_infos.config.conf_cl[CF_MAX_CON];
  del = G_cctde_infos.config.conf_cl_dal[CF_MAX_CON];
  per = G_cctde_infos.config.conf_cl_pal[CF_MAX_CON];

  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->tcp_nb,max,
					     ALERT_STATS_CON,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_CL stats : ",get_str_from_time_t(current_time()));
      display_infos(stdout,"Conn. reached %d : %s openned %d connections",max,ip_uint_to_str(stored_view->ip),stored_view->tcp_nb);
      if (ret == 0) display_infos(stdout,".\n");
      if (ret == 1) display_infos(stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos(stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_cl[CF_MAX_UPTIME];
  del = G_cctde_infos.config.conf_cl_dal[CF_MAX_UPTIME];
  per = G_cctde_infos.config.conf_cl_pal[CF_MAX_UPTIME];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->com_uptime,max,
				 ALERT_STATS_UPTIME,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_CL stats : ",get_str_from_time_t(current_time()));
      display_infos(stdout,"Global com uptime reached %d seconds : %s stood up for %d secs.\n",max,ip_uint_to_str(stored_view->ip),stored_view->com_uptime);
      if (ret == 0) display_infos(stdout,".\n");
      if (ret == 1) display_infos(stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos(stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_cl[CF_MAX_PRX];
  del = G_cctde_infos.config.conf_cl_dal[CF_MAX_PRX];
  per = G_cctde_infos.config.conf_cl_pal[CF_MAX_PRX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->packet_rx,max,
				 ALERT_STATS_PRX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_CL stats : ",get_str_from_time_t(current_time()));
      display_infos(stdout,"Packet rx reached %d : %s received %d packets",max,ip_uint_to_str(stored_view->ip),stored_view->packet_rx);
      if (ret == 0) display_infos(stdout,".\n");
      if (ret == 1) display_infos(stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos(stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_cl[CF_MAX_PTX];
  del = G_cctde_infos.config.conf_cl_dal[CF_MAX_PTX];
  per = G_cctde_infos.config.conf_cl_pal[CF_MAX_PTX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->packet_tx,max,
				 ALERT_STATS_PTX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_CL stats : ",get_str_from_time_t(current_time()));
      display_infos(stdout,"Packet tx reached %d : %s sent %d packets",max,ip_uint_to_str(stored_view->ip),stored_view->packet_tx);
      if (ret == 0) display_infos(stdout,".\n");
      if (ret == 1) display_infos(stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos(stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_cl[CF_MAX_BRX];
  del = G_cctde_infos.config.conf_cl_dal[CF_MAX_BRX];
  per = G_cctde_infos.config.conf_cl_pal[CF_MAX_BRX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->bytes_rx,max,
				 ALERT_STATS_BRX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_CL stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Bytes rx reached %d : %s received %d bytes",max,ip_uint_to_str(stored_view->ip),stored_view->bytes_rx);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_cl[CF_MAX_BTX];
  del = G_cctde_infos.config.conf_cl_dal[CF_MAX_BTX];
  per = G_cctde_infos.config.conf_cl_pal[CF_MAX_BTX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->bytes_tx,max,
				 ALERT_STATS_BTX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_CL stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Bytes tx reached %d : %s sent %d bytes",max,ip_uint_to_str(stored_view->ip),stored_view->bytes_tx);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  /* Take care to return 0 to browse the whole list */
  return (0);
}

int basic_server_stats(t_storage *stored_view, t_storage *null_ptr)
{
  int ret;
  /* Maximum threshold, Delay before remind, percentage before remind... */
  u_int32_t max,del,per; 

  max = G_cctde_infos.config.conf_srv[CF_MAX_CON];
  del = G_cctde_infos.config.conf_srv_dal[CF_MAX_CON];
  per = G_cctde_infos.config.conf_srv_pal[CF_MAX_CON];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->tcp_nb,max,
				 ALERT_STATS_CON,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_SRV stats : ",get_str_from_time_t(current_time()));
      display_infos(stdout,"Conn. reached %d : %s got %d connections",max,ip_uint_to_str(stored_view->ip),stored_view->tcp_nb);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_srv[CF_MAX_UPTIME];
  del = G_cctde_infos.config.conf_srv_dal[CF_MAX_UPTIME];
  per = G_cctde_infos.config.conf_srv_pal[CF_MAX_UPTIME];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->com_uptime,max,
				 ALERT_STATS_UPTIME,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_SRV stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Global com uptime reached %d seconds : %s stood up for %d secs.\n",max,ip_uint_to_str(stored_view->ip),stored_view->com_uptime);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_srv[CF_MAX_PRX];
  del = G_cctde_infos.config.conf_srv_dal[CF_MAX_PRX];
  per = G_cctde_infos.config.conf_srv_pal[CF_MAX_PRX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->packet_rx,max,
				 ALERT_STATS_PRX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_SRV stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Packet rx reached %d : %s received %d packets",max,ip_uint_to_str(stored_view->ip),stored_view->packet_rx);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_srv[CF_MAX_PTX];
  del = G_cctde_infos.config.conf_srv_dal[CF_MAX_PTX];
  per = G_cctde_infos.config.conf_srv_pal[CF_MAX_PTX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->packet_tx,max,
				 ALERT_STATS_PTX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_SRV stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Packet tx reached %d : %s sent %d packets",max,ip_uint_to_str(stored_view->ip),stored_view->packet_tx);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_srv[CF_MAX_BRX];
  del = G_cctde_infos.config.conf_srv_dal[CF_MAX_BRX];
  per = G_cctde_infos.config.conf_srv_pal[CF_MAX_BRX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->bytes_rx,max,
				 ALERT_STATS_BRX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_SRV stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Bytes rx reached %d : %s received %d bytes",max,ip_uint_to_str(stored_view->ip),stored_view->bytes_rx);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  max = G_cctde_infos.config.conf_srv[CF_MAX_BTX];
  del = G_cctde_infos.config.conf_srv_dal[CF_MAX_BTX];
  per = G_cctde_infos.config.conf_srv_pal[CF_MAX_BTX];
  if (max != 0 && (ret = generic_basic_stats(stored_view,stored_view->bytes_tx,max,
				 ALERT_STATS_BTX,del,per)) >= 0)
    {
      display_infos(stdout,"[%s] CCTDE - MAX_SRV stats : ",get_str_from_time_t(current_time()));
      display_infos (stdout,"Bytes tx reached %d : %s sent %d bytes",max,ip_uint_to_str(stored_view->ip),stored_view->bytes_tx);
      if (ret == 0) display_infos (stdout,".\n");
      if (ret == 1) display_infos (stdout," (rem. %d secs).\n",del);
      if (ret == 2) display_infos (stdout," (rem. %d%%).\n",per);
    }

  /* Take care to return 0 to browse the whole list */
  return (0);
}
