/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"


/*
** Detecting suspect flows on current connections...
*/

int upcon_detect_uptime(t_con_track *ptr)
{
  int runtime;
  int testval;

  if ((testval = G_cctde_infos.config.conf_upcon[CF_MAX_UPCON_UPTIME]) == 0)
    return (0);

  if (G_cctde_infos.start_time_pcap != 0)
    runtime = (int) (ptr->time_last_access.tv_sec - ptr->time_first_access.tv_sec);
  else
    runtime = (int) running_time(ptr->time_first_access.tv_sec,0);
  
  if (runtime > testval)
    {
      display_infos (stdout,"CCTDE : Con created more than %d seconds ago : ",testval);
      display_flow(stdout,ptr->flow_infos);
      display_infos (stdout," (%d seconds).\n",runtime);
    }
  return (1);
}

int upcon_detect_packets_amount(t_con_track *ptr)
{
  int testval;

  if (ptr->packet_rx > (testval = G_cctde_infos.config.conf_upcon[CF_MAX_UPCON_PRX]))
    {
      if (testval == 0)
	return (0);
      display_infos (stdout,"CCTDE : Connections with more than %d RX packets : ",testval);
      display_flow(stdout,ptr->flow_infos);
      display_infos (stdout," received %d packets.\n",ptr->packet_rx);
    }

  if (ptr->packet_tx > (testval = G_cctde_infos.config.conf_upcon[CF_MAX_UPCON_PTX]))
    {
      if (testval == 0)
	return (0);
      display_infos (stdout,"CCTDE : Connections with more than %d TX packets : ",testval);
      display_flow(stdout,ptr->flow_infos);
      display_infos (stdout," sent %d packets.\n",ptr->packet_tx);
    }
  return (1);
}

int upcon_detect_bytes_amount(t_con_track *ptr)
{
  int testval;

  if (ptr->bytes_rx > (testval = G_cctde_infos.config.conf_upcon[CF_MAX_UPCON_BRX]))
    {
      if (testval == 0)
	return (0);
      display_infos (stdout,"CCTDE : Connections with more than %d RX bytes : ",testval);
      display_flow(stdout,ptr->flow_infos);
      display_infos (stdout," received %d bytes.\n",ptr->bytes_rx);
    }

  if (ptr->bytes_tx > (testval = G_cctde_infos.config.conf_upcon[CF_MAX_UPCON_BTX]))
    {
      if (testval == 0)
	return (0);
      display_infos (stdout,"CCTDE : Connections with more than %d TX bytes : ",testval);
      display_flow(stdout,ptr->flow_infos);
      display_infos (stdout," sent %d bytes.\n",ptr->bytes_tx);
    }
  return (1);
}
