/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** These functions belong to and are updated from the Gray-world CCTT project.
*/

#include "../includes/cctde.h"

typedef struct s_exec_cmd_file {
  char *label;
  int (*fn)();
  int flag;
} t_exec_cmd_file;

int exec_search(char *function, char *params, int flag)
{
  t_storage tmp_view;
  t_list_elem *tmp_elem;

  if (!params || strlen(params) < 1)
    return (0);
  if (flag == 0)
    {
      if ((tmp_view.ip = get_uint_ip_from_str(params)) == 0)
	return (0);
      if (!(tmp_elem = look_for_elem(&(G_cctde_infos.client_list),&tmp_view,match_ip)))
	{
	  display_infos (stdout,"%s : %s doesn't exist.\n",function,params);
	  return (1);
	}
      else
	{
	  display_clients(tmp_elem->data);
	  return (1);
	}
    }
  return (0);
}

int exec_display(char *function, char *params, int flag)
{
  if (flag == 0)
    {
      display_top_servers();
      return (1);
    }
  if (flag == 1)
    {
      display_top_clients();
      return (1);
    }
  if (flag == 2)
    {
      display_infos(stdout,"  o %d recorded client(s).\n",G_cctde_infos.client_list.elem_nb);
      browse_existing_elems(&(G_cctde_infos.client_list),display_clients);
      return (1);
    }
  if (flag == 3)
    {
      display_infos(stdout,"  o %d recorded client(s).\n",G_cctde_infos.client_list.elem_nb);
      browse_existing_elems(&(G_cctde_infos.client_list),display_clients_and_servers);
      return (1);
    }
  if (flag == 4)
    {
      display_infos(stdout,"  o %d recorded server(s).\n",G_cctde_infos.server_list.elem_nb);
      browse_existing_elems(&(G_cctde_infos.server_list),display_servers);
      return (1);
    }
  return (0);
}

/*
** If no top exists, then build it from scratch
** If top exists and TopX > current_val then only change the TopX val
**   keeping recorded elements.
** If top exists but TopX < current_val then parse the whole list and
**   update Top.
*/

typedef struct s_conf_topX {
  char *label;
  int pos;
  t_list_main *lm_cl;
  int (*fn1_cl)();
  int (*fn2_cl)();
  t_list_main *lm_srv;
  int (*fn1_srv)();
  int (*fn2_srv)();
} t_conf_topX;

int build_topX(char *function, char *params, int flag)
{
  int i,topX;
  static char type[MAX_LABEL];
  static char val[MAX_VAL];
  t_list_elem *tmp;

  static t_conf_topX conf_topX[]= {
    { "con", CF_MAX_CON , 
      &G_cctde_infos.TOPX_CL_con, match_ip, wgrteq_con,
      &G_cctde_infos.TOPX_SRV_con, match_ipport, wgrteq_con 
    },
    { "uptime", CF_MAX_UPTIME ,
      &G_cctde_infos.TOPX_CL_uptime, match_ip, wgrteq_uptime,
      &G_cctde_infos.TOPX_SRV_uptime, match_ipport, wgrteq_uptime 
    },
    { "prx", CF_MAX_PRX ,
      &G_cctde_infos.TOPX_CL_prx, match_ip, wgrteq_prx,
      &G_cctde_infos.TOPX_SRV_prx, match_ipport, wgrteq_prx 
    },
    { "ptx", CF_MAX_PTX ,
      &G_cctde_infos.TOPX_CL_ptx, match_ip, wgrteq_ptx,
      &G_cctde_infos.TOPX_SRV_ptx, match_ipport, wgrteq_ptx 
    },
    { "brx", CF_MAX_BRX ,
      &G_cctde_infos.TOPX_CL_brx, match_ip, wgrteq_brx,
      &G_cctde_infos.TOPX_SRV_brx, match_ipport, wgrteq_brx 
    },
    { "btx", CF_MAX_BTX ,
      &G_cctde_infos.TOPX_CL_btx, match_ip, wgrteq_btx,
      &G_cctde_infos.TOPX_SRV_btx, match_ipport, wgrteq_btx 
    },
    { NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL }
  };

  if (!params || strlen(params) < 1)
    return (0);

  for(i=0 ; params[i] && params[i] != ';' && i < MAX_LABEL; i++)
    ;
  if (i == 0 || i == (strlen(params)-1) || 
      i > MAX_LABEL-1 || strlen(params) > i+MAX_VAL)
    return (0);

  memset(type,0,MAX_LABEL);
  memset(val,0,MAX_VAL);
  strncpy(type,params,i);
  strncpy(val,params+i+1,strlen(params)-i);

  for (i=0;conf_topX[i].label != NULL;i++)
    if (strlen(conf_topX[i].label) == strlen(type) &&
	strncmp(conf_topX[i].label,type,strlen(type)) == 0)
      {
	topX = (u_int32_t) atoi(val);

	if (flag == 0) {

	  if (G_cctde_infos.config.conf_topX_cl[conf_topX[i].pos] >= topX) {
	    G_cctde_infos.config.conf_topX_cl[conf_topX[i].pos] = topX;
	    return (1);
	  }

	  G_cctde_infos.config.conf_topX_cl[conf_topX[i].pos] = topX;
	  if ((tmp = G_cctde_infos.client_list.first)) {
	    if (tmp->data)
	      do {
		store_top_X(tmp,conf_topX[i].lm_cl,
			    conf_topX[i].fn1_cl,conf_topX[i].fn2_cl,topX);
	      } while ((tmp = tmp->next) != NULL && tmp->data);
	  }
	  return (1);
	}

	if (flag == 1) {

	  if (G_cctde_infos.config.conf_topX_srv[conf_topX[i].pos] >= topX) {
	    G_cctde_infos.config.conf_topX_srv[conf_topX[i].pos] = topX;
	    return (1);
	  }

	  G_cctde_infos.config.conf_topX_srv[conf_topX[i].pos] = topX;
	  if ((tmp = G_cctde_infos.server_list.first)) {
	    if (tmp->data)
	      do {
		store_top_X(tmp,conf_topX[i].lm_srv,
			    conf_topX[i].fn1_srv,conf_topX[i].fn2_srv,topX);
	      } while ((tmp = tmp->next) != NULL && tmp->data);
	  }
	  return (1);
	}
      }
  return (0);
}

static t_exec_cmd_file exec_cmd_file[] = {
  { "display_top_servers" , exec_display , 0 },
  { "display_top_clients" , exec_display , 1 },
  { "display_clients_infos" , exec_display , 2 },
  { "display_clients_and_servers_infos" , exec_display , 3 },
  { "display_servers_infos" , exec_display , 4 } ,

  { "display_client_x", exec_search , 0 },

  { "build_topX_clients", build_topX , 0},
  { "build_topX_servers", build_topX , 1},

  /* In backup.c */
  { "backup_stats_to", backup_cctde_infos , 0},
  { "read_stats_from", read_stats_from , 0},

  { NULL, NULL, 0 }
};

int execute_cmd_file(char *function,char *params)
{
  int i;

  for (i=0;exec_cmd_file[i].label != NULL;i++)
    if (strncmp(exec_cmd_file[i].label,function,strlen(function)) == 0)
      {
	display_infos(stdout,"%s : %s\n",function,params);
	if ((exec_cmd_file[i].fn(function,params,exec_cmd_file[i].flag)) == 0)
	  display_infos(stderr,"In %s, %s=%s is not valid...\n",
			G_cctde_infos.config.cmd_file,
			function,params);
	  return (1);
      }
  display_infos(stderr,"In %s, %s is not valid...\n",G_cctde_infos.config.cmd_file,function);
  return (0);
}
