/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

/*
** Browse list elem after elem and use the seek_function to
** match pattern in stored data.
*/

void *look_for_elem(t_list_main *lm, void *data,int (*seek_function)())
{
  t_list_elem *tmp;

  if (!(tmp = lm->first))
    return (NULL);
  do {
    if (tmp->data && seek_function(tmp->data,data) == 1)
      return (tmp);
  } while ((tmp = tmp->next) != NULL);
  return (NULL);
}

void *reverse_look_for_elem(t_list_main *lm, void *data,int (*seek_function)())
{
  t_list_elem *tmp;

  if (!(tmp = lm->last))
    return (NULL);
  do {
    if (tmp->data && seek_function(tmp->data,data) == 1)
      return (tmp);
  } while ((tmp = tmp->prev) != NULL);
  return (NULL);
}

void *look_for_elem_and_count(t_list_main *lm, void *data,int (*seek_function)(),int stop)
{
  t_list_elem *tmp;
  int i=0;

  if (!(tmp = lm->first))
    return (NULL);
  do {
    if (stop != 0 && i < stop)
      i++;
    else
      if (stop != 0)
	return (NULL);
    if (tmp->data && seek_function(tmp->data,data) == 1)
      return (tmp);
  } while ((tmp = tmp->next) != NULL);
  return (NULL);
}

/*
** Replace an element by another
*/

t_list_elem *replace_elem(t_list_main *lm, t_list_elem *old, t_list_elem *new)
{
  if (!(lm) || !(old) || !(new))
    return (NULL);

  new->prev = old->prev;
  new->next = old->next;
  if (old->prev && old->prev->next)
    old->prev->next = new;
  if (old->next && old->next->prev)
    old->next->prev = new;

  if (lm->first == old) lm->first = new;
  if (lm->last == old) lm->last = new;
  if (lm->ladded == old) lm->ladded = new;
  if (lm->lupdated == old) lm->lupdated = new;
  old = new;
  return (new);
}

/*
** Remove an element
** If addtype == 0, only remove the ptr addr.
** If addtype == 1, free the elem. 
*/

int remove_elem(t_list_main *lm, t_list_elem *ptr, int remtype)
{
  if (!(lm) || !(ptr))
    return (-1);

  if (ptr->prev && ptr->next)
    {
      ptr->prev->next = ptr->next;
      ptr->next->prev = ptr->prev;
    }
  else
    {
      if (ptr->prev)
	ptr->prev->next = NULL;
      if (ptr->next)
	ptr->next->prev = NULL;
    }

  if (lm->first == ptr) lm->first = ptr->next;
  if (lm->last == ptr) lm->last = ptr->prev;
  if (lm->ladded == ptr) lm->ladded = NULL;
  if (lm->lupdated == ptr) lm->lupdated = NULL;
  lm->elem_nb--;
  if (remtype == 1)
    free (ptr);
  return (1);
}

/*
** Add data to a new list element
** If addtype == 0, only get the ptr addr.
** If addtype == 1, memcpy the elem. 
*/

t_list_elem *add_to_list_here(t_list_main *lm, void *data, u_int32_t sizeofdata,t_list_elem *ptr_next, int addtype)
{
  t_list_elem *current;

  if ((current = (t_list_elem *) malloc(sizeof(t_list_elem))) == NULL)
    return (NULL);
  
  current->sizeofdata = sizeofdata;
  if ((current->data = (void *) malloc(current->sizeofdata)) == NULL)
    {
      free (current);
      return (NULL);
    }

  if (addtype == 1)
    memcpy(current->data,data,current->sizeofdata);
  else
    current->data = data;
  
  if (ptr_next == NULL) // LAST
    {
      lm->last->next = current;
      current->prev = lm->last;
      current->next = NULL;
      lm->last = current;
    }
  else
    {
      current->next = ptr_next;
      if (ptr_next->prev) // MIDDLE
	{
	  current->prev = ptr_next->prev;
	  ptr_next->prev->next = current;
	  current->next->prev = current;
	}
      else // FIRST
	{
	  current->prev = NULL;
	  current->next->prev = current;
	  lm->first = current;
	}
    }

  lm->ladded = current;
  lm->elem_nb++;
  return (current);
}

/*
** Update an element stored in the list using the update_function
*/

t_list_elem *update_elem(t_list_elem *elem, void *data,int (*update_function)())
{
  if (!(elem))
    return (NULL);
  if (elem->data && update_function(elem->data,data) == 1)
      return (elem);
  return (NULL);
}

/*
** Add data to a new list element
** If addtype == 0, only get the ptr addr.
** If addtype == 1, memcpy the elem. 
*/

t_list_elem *add_to_list(t_list_main *lm, void *data, u_int32_t sizeofdata,int addtype)
{
  t_list_elem *current;

  if ((current = (t_list_elem *) malloc(sizeof(t_list_elem))) == NULL)
    return (NULL);
  
  current->sizeofdata = sizeofdata;
  if ((current->data = (void *) malloc(current->sizeofdata)) == NULL)
    {
      free (current);
      return (NULL);
    }

  if (addtype == 1)
    memcpy(current->data,data,current->sizeofdata);
  else
    current->data = data;
  
  current->prev = NULL;
  if (!lm->first) {
    current->next = NULL;
    lm->first = current;
    lm->last = current;
    lm->ladded = current;
    lm->elem_nb = 1;
    lm->sizeof_elem = current->sizeofdata;
  } else {
    (lm->first)->prev = current;
    current->next = lm->first;
    lm->first = current;
    lm->ladded = current;
    lm->elem_nb++;
  }
  return (current);
}

/*
** Browse the list elem after elem and display data
*/

void browse_existing_elems(t_list_main *lm, void (*display_function)())
{
  t_list_elem *tmp;

  if ((tmp = lm->first)) {
    if (tmp->data)
      display_function(tmp->data);
    while ((tmp = tmp->next) != NULL && tmp->data)
      display_function(tmp->data);
  }
}

void browse_existing_elems_until(t_list_main *lm, void (*display_function)(), int until_this)
{
  t_list_elem *tmp;
  int go_on=1;
  int cpt=1;

  if ((tmp = lm->first)) {
    if (tmp->data)
      do {
	display_function(tmp->data);
	if ((tmp = tmp->next) == NULL || !(tmp->data))
	  go_on = 0;
	if (cpt == until_this && until_this != 0)
	  go_on = 0;
	else
	  cpt+=1;
      } while (go_on == 1);
  }
}

/*
** Debug function / don't use it...
*/

void debug_list(t_list_main lm)
{
  t_list_elem *tmp;

  fprintf (stderr,"  Lm has %d elems : First is at 0x%x - Last is at 0x%x - Ladded is at 0x%x - Lupdated is 0x%x\n  Browsing :\n", lm.elem_nb,(int) lm.first,(int) lm.last, (int) lm.ladded, (int) lm.lupdated);

  if ((tmp = lm.first)) {
    fprintf (stderr,"    # Current is 0x%x - next is 0x%x - prev is 0x%x - data is at 0x%x.\n",
	     (int) tmp, (int) tmp->next, (int) tmp->prev, (int) tmp->data);
    while ((tmp = tmp->next) != NULL)
      fprintf (stderr,"    # Current is 0x%x - next is 0x%x - prev is 0x%x - data is at 0x%x.\n",
               (int) tmp, (int) tmp->next, (int) tmp->prev, (int) tmp->data);
  }
}

void init_main_list(t_list_main *list_main)
{
  list_main->elem_nb = 0;
  list_main->sizeof_elem = 0;

  list_main->first = NULL;
  list_main->last = NULL;
  list_main->ladded = NULL;
  list_main->lupdated = NULL;
}
