/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

#define STORAGE_BUILD_FOR_CL 0
#define STORAGE_BUILD_FOR_SRV 1

t_storage build_storage_view(t_workflow workflow,Alertpkt alert,Packet *p,int stype)
{
  t_storage storage_view;
  u_int data_size;

  data_size= ntohs(p->iph->ip_len) -
    ((IP_HLEN(p->iph) << 2) + (TCP_OFFSET(p->tcph) << 2));

  bzero(&storage_view,sizeof(t_storage));

  if (stype == STORAGE_BUILD_FOR_SRV)
    {
      storage_view.magic_s = STORAGE_SRV_MAGIC_S;
      storage_view.magic_e = STORAGE_SRV_MAGIC_E;
      storage_view.ip = (workflow.ptr)->flow_infos.dst;
      storage_view.port = (workflow.ptr)->flow_infos.dport;
    }
  else
    {
      storage_view.magic_s = STORAGE_CL_MAGIC_S;
      storage_view.magic_e = STORAGE_CL_MAGIC_E;
      storage_view.ip = (workflow.ptr)->flow_infos.src;
      storage_view.port = (workflow.ptr)->flow_infos.sport;
      storage_view.sec_list_qt = 1;
    }
  storage_view.time_last_access = p->pkth->ts;

  if (FLOW_WAY(workflow.ptr,p) == 0)
    {
      storage_view.packet_tx = 1;
      storage_view.bytes_tx = data_size;
    }
  else
    {
      storage_view.packet_rx = 1;
      storage_view.bytes_rx = data_size;
    }

  /* 
  ** Only used for the first time as these fields are not taken
  ** in consideration in the update_stored_view function...
  */
  storage_view.tcp_nb = 1;
  storage_view.time_creation = p->pkth->ts;
  storage_view.time_first_access = p->pkth->ts;

  return (storage_view);
}

int match_timeperiod(t_time_storage *stored, struct timeval *timeval)
{
  time_t current_period;
  u_int32_t period;

  current_period = stored->time_creation;
  period = stored->period;

  if (timeval->tv_sec >= current_period &&
      timeval->tv_sec < (current_period + period))
    return (1);
  return (-1);
}

int match_ip(t_storage *stored_view, t_storage *tmp_view)
{
  if (stored_view->ip == tmp_view->ip)
    return(1);
  return(-1);
}

int match_ipport(t_storage *stored_view, t_storage *tmp_view)
{
  if (stored_view->ip == tmp_view->ip &&
      stored_view->port == tmp_view->port)
    return(1);
  return(-1);
}

int update_stored_view(t_storage *stored_view, t_storage *tmp_view)
{
  stored_view->packet_rx += tmp_view->packet_rx;
  stored_view->packet_tx += tmp_view->packet_tx;
  stored_view->bytes_rx += tmp_view->bytes_rx;
  stored_view->bytes_tx += tmp_view->bytes_tx;
  stored_view->time_last_access = tmp_view->time_last_access;
  return (1);
}

t_list_elem *store_flow(t_workflow workflow, Alertpkt alert,Packet *p)
{
  t_storage client_view,server_view,*tmp_view;
  t_list_elem *ptr,*ptr2,*ptr3; // ptr = client , ptr2 = client.server , ptr3 = server

  u_int32_t tmp;

  client_view = build_storage_view(workflow,alert,p,STORAGE_BUILD_FOR_CL);
  server_view = build_storage_view(workflow,alert,p,STORAGE_BUILD_FOR_SRV);

  /* We now use the list framework to store infos for the client */

  /* 
  ** We have two ways to get the pointer to the current client infos :
  ** 1/ we browse the whole list to find it.
  ** 2/ we use the previously ptr saved in the con_track related flow
  ** I'm choosing 2/ so that clients which are between two timeline
  ** periods belong to the period they initiated their connection.
  ** 
  ** If you know want keeping packets stats to the period they belong
  ** remove the first next statement.
  */

  if (!(ptr = (workflow.ptr)->infos))
    ptr = look_for_elem(&(G_cctde_infos.client_list),&client_view,match_ip);

  if (ptr)
    {
      /* 
      ** Element previously existed but this is a new connection and next
      ** values are not changed in the update function.
      */
      if ((workflow.ptr)->state == FLOW_STATE_ON_CONNECT)
	{
	  tmp_view = (t_storage *) ptr->data;
	  tmp_view->tcp_nb ++;
	  tmp_view->time_first_access = p->pkth->ts;
	}
      if (!(ptr = update_elem(ptr,&client_view,update_stored_view)))
	return (NULL);
    }
  else
    {
      if ((ptr = add_to_list(&(G_cctde_infos.client_list),&client_view,sizeof(t_storage),1)) == NULL)
	return (NULL);
    }

  if ((ptr3 = look_for_elem(&(G_cctde_infos.server_list),&server_view,match_ipport)))
    {
      /* 
      ** Element previously existed but this is a new connection and next
      ** values are not changed in the update function.
      */
      if ((workflow.ptr)->state == FLOW_STATE_ON_CONNECT)
	{
	  tmp_view = (t_storage *) ptr3->data;
	  tmp_view->tcp_nb ++;
	  tmp_view->time_first_access = p->pkth->ts;
	}
      if (!(ptr3 = update_elem(ptr3,&server_view,update_stored_view)))
	return (NULL);
    }
  else
    {
      if ((ptr3 = add_to_list(&(G_cctde_infos.server_list),&server_view,sizeof(t_storage),1)) == NULL)
	return (NULL);
    }

  //  debug_list(G_cctde_infos.client_list);

  /* 
  ** Now that the client infos are stored, we update the related
  ** client with its server informations...
  */

  tmp_view = (t_storage *) ptr->data;
  if ((ptr2 = look_for_elem(&(tmp_view->second_list),&server_view,match_ipport)))
    {
      /* Element previously existed but this is a new connection */
      if ((workflow.ptr)->state == FLOW_STATE_ON_CONNECT)
	{
	  tmp_view = (t_storage *) ptr2->data;
	  tmp_view->tcp_nb ++;
	  tmp_view->time_first_access = p->pkth->ts;
	}
      if (!(ptr2 = update_elem(ptr2,&server_view,update_stored_view)))
	return (NULL);
    }
  else
    {
      if ((ptr2 = add_to_list(&(tmp_view->second_list),&server_view,sizeof(t_storage),1)) == NULL)
	return (NULL);
      /* Keep a pointer to our server global infos */
      tmp_view = (t_storage *) ptr2->data;
      tmp_view->server_infos = ptr3;
      tmp_view->sec_list_qt += 1;
    }

  if ((tmp = G_cctde_infos.config.conf_topX_cl[CF_MAX_CON]) != 0)
    store_top_X(ptr,&(G_cctde_infos.TOPX_CL_con),match_ip,wgrteq_con,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_cl[CF_MAX_UPTIME]) != 0)
    store_top_X(ptr,&(G_cctde_infos.TOPX_CL_uptime),match_ip,wgrteq_uptime,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_cl[CF_MAX_PRX]) != 0)
    store_top_X(ptr,&(G_cctde_infos.TOPX_CL_prx),match_ip,wgrteq_prx,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_cl[CF_MAX_PTX]) != 0)
    store_top_X(ptr,&(G_cctde_infos.TOPX_CL_ptx),match_ip,wgrteq_ptx,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_cl[CF_MAX_BRX]) != 0)
    store_top_X(ptr,&(G_cctde_infos.TOPX_CL_brx),match_ip,wgrteq_brx,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_cl[CF_MAX_BTX]) != 0)
    store_top_X(ptr,&(G_cctde_infos.TOPX_CL_btx),match_ip,wgrteq_btx,tmp);

  if ((tmp = G_cctde_infos.config.conf_topX_srv[CF_MAX_CON]) != 0)
    store_top_X(ptr3,&(G_cctde_infos.TOPX_SRV_con),match_ipport,wgrteq_con,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_srv[CF_MAX_UPTIME]) != 0)
    store_top_X(ptr3,&(G_cctde_infos.TOPX_SRV_uptime),match_ipport,wgrteq_uptime,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_srv[CF_MAX_PRX]) != 0)
    store_top_X(ptr3,&(G_cctde_infos.TOPX_SRV_prx),match_ipport,wgrteq_prx,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_srv[CF_MAX_PTX]) != 0)
    store_top_X(ptr3,&(G_cctde_infos.TOPX_SRV_ptx),match_ipport,wgrteq_ptx,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_srv[CF_MAX_BRX]) != 0)
    store_top_X(ptr3,&(G_cctde_infos.TOPX_SRV_brx),match_ipport,wgrteq_brx,tmp);
  if ((tmp = G_cctde_infos.config.conf_topX_srv[CF_MAX_BTX]) != 0)
    store_top_X(ptr3,&(G_cctde_infos.TOPX_SRV_btx),match_ipport,wgrteq_btx,tmp);

  /* Keep a pointer to last acceded server elem */
  tmp_view = (t_storage *) ptr->data;
  tmp_view->sec_list_last_acceded = (t_list_elem *) ptr2;
  return (ptr);
}

u_int32_t get_start_period()
{
  t_list_elem *ptr_time;
  t_time_storage *time_stored;
  u_int32_t current_start_period;

  /* We don't have this before first rotation */
  if ((ptr_time = (t_list_elem *) G_cctde_infos.time_list.ladded))
    {
      time_stored = (t_time_storage *) ptr_time->data;
      current_start_period = time_stored->time_creation;
    }
  else
    {
      if (G_cctde_infos.start_time_pcap != 0)
	current_start_period = G_cctde_infos.start_time_pcap;
      else
	current_start_period = G_cctde_infos.start_time;
    }
  return (current_start_period);
}

t_list_elem *consider_timeline(t_workflow workflow, Alertpkt alert,Packet *p)
{
  t_list_elem *ptr_time;
  t_time_storage *time_stored,new_time_storage;
  u_int32_t current_start_period;

  if (CONSIDER_TIMELINE() == 0)
    return(store_flow(workflow,alert,p));

  current_start_period = get_start_period();

  if (p->pkth->ts.tv_sec > current_start_period) // After or before current period ?
    {
      if ((p->pkth->ts.tv_sec - current_start_period) < ROTATION_PERIOD())
	{
	  // This packet belongs to the current period
	  return(store_flow(workflow,alert,p));
	}
      else 
	{
	  // This packet belongs to a next period
	  // We create empty time elements until we get the period-rotation_period previous elem
	  while (get_start_period() + (ROTATION_PERIOD()*2) < p->pkth->ts.tv_sec)
	    {
	      memset(&new_time_storage,0,sizeof(t_time_storage));
	      if (!(ptr_time = (t_list_elem *) G_cctde_infos.time_list.ladded))
		{
		  // Going here the first time only
		  new_time_storage.time_creation = current_start_period;
		  new_time_storage.period = ROTATION_PERIOD();
		}
	      else
		{
		  time_stored = (t_time_storage *) ptr_time->data;
		  new_time_storage.time_creation = 
		    time_stored->time_creation+time_stored->period;
		  new_time_storage.period = ROTATION_PERIOD();
		}
	      new_time_storage.magic_s = STORAGE_TP_MAGIC_S;
	      new_time_storage.magic_e = STORAGE_TP_MAGIC_E;

	      // Only store something if there is recorded clients
	      if (G_cctde_infos.client_list.elem_nb > 0)
		{
		  new_time_storage.packets_number = G_cctde_infos.packets_number;
		  new_time_storage.tcp_conn_number = G_cctde_infos.tcp_conn_number;
		  new_time_storage.data_amount = G_cctde_infos.data_amount;
		  new_time_storage.client_number = G_cctde_infos.client_list.elem_nb;
		  memcpy(&new_time_storage.client_list,&G_cctde_infos.client_list,sizeof(t_list_main));
		}

	      if (add_to_list(&(G_cctde_infos.time_list),
			      &new_time_storage,sizeof(t_time_storage),1) == NULL)
		display_infos (stderr,"Couldn't add the new client list after the %d rotation period !\n",
			       ROTATION_PERIOD());
	      else
		init_main_list(&G_cctde_infos.client_list);
	    }
	}
    }
  else // This packet belongs to a previous period
    {
      if (!(ptr_time = look_for_elem(&G_cctde_infos.time_list,&(p->pkth->ts),match_timeperiod)))
	{
	  if (G_cctde_infos.time_list.ladded && !(ptr_time = (workflow.ptr)->infos))
	    display_infos(stderr,"Packet belongs to a previous period and no flow exists !?!?\n");
	  // else belongs to non existing but flow is shared
	}
      // else belongs to previous existing period and will be added in...
    }
  return(store_flow(workflow,alert,p));
}
