/*
** CCTDE - Covert Channels and Tunnels Detection Engine - v0.1
** Copyright (C) 2003,2004 Simon Castro - scastro@entreelibre.com
**
** This file is part of CCTDE - v0.1 (C) 2003,2004 Simon Castro.
** 
** Cctde is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctde is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctde; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/cctde.h"

/*
** Keeping track of TCP connections
*/

t_con_track *G_con_track[CON_HASH_SIZE];

void init_con_track()
{
  int i;

  for(i=0; i<CON_HASH_SIZE; i++) 
    G_con_track[i] = NULL;
}

u_int32_t get_tcp_flow_infos(t_flow *flow_infos, Packet *p)
{
  flow_infos->src = ntohl(p->iph->ip_src.s_addr);
  flow_infos->dst = ntohl(p->iph->ip_dst.s_addr);
  flow_infos->sport = ntohs(p->tcph->th_sport);
  flow_infos->dport = ntohs(p->tcph->th_dport);
  return (HASH_FLOW(flow_infos));
}

t_flow get_tcp_rev_flow_infos(t_flow flow_infos)
{
  t_flow rev_flow_infos;

  rev_flow_infos.src = flow_infos.dst;
  rev_flow_infos.dst = flow_infos.src;
  rev_flow_infos.sport = flow_infos.dport;
  rev_flow_infos.dport = flow_infos.sport;

  return (rev_flow_infos);
}

void build_workflow(t_workflow *workflow, Packet *p)
{
  t_flow flow_infos;
  workflow->flow_hash = get_tcp_flow_infos(&flow_infos,p);
  workflow->flow_infos = flow_infos;
  workflow->ptr = NULL;
}

t_con_track *add_con_track_flow(t_workflow *workflow)
{
  t_con_track *new_con_track;

  if ((new_con_track = (t_con_track *) malloc (sizeof(t_con_track))) == NULL)
    return (NULL);

  bzero (new_con_track,sizeof(t_con_track));
  memcpy(&new_con_track->flow_infos,&workflow->flow_infos,sizeof(t_flow));
  new_con_track->state = FLOW_STATE_ON_CONNECT;
  if (G_con_track[workflow->flow_hash])
    G_con_track[workflow->flow_hash]->prev = new_con_track;
  new_con_track->prev = NULL;
  new_con_track->next = G_con_track[workflow->flow_hash];
  G_con_track[workflow->flow_hash] = new_con_track;

  G_cctde_infos.tcp_conn_number += 1;
  G_cctde_infos.tcp_conn_alive_number += 1;

  workflow->ptr = new_con_track;
  return (new_con_track);
}

t_con_track *find_con_track_flow(t_workflow *workflow)
{
  t_con_track *ptr;
  t_flow flow_infos,rev_flow_infos;
  u_int32_t flow_hash;

  flow_infos = workflow->flow_infos;
  rev_flow_infos = get_tcp_rev_flow_infos(workflow->flow_infos);
  flow_hash = workflow->flow_hash;

  for (ptr = G_con_track[flow_hash]; ptr != NULL; ptr = ptr->next)
    {
      if (memcmp(&flow_infos,&(ptr->flow_infos),sizeof(t_flow)) ||
	  memcmp(&rev_flow_infos,&(ptr->flow_infos),sizeof(t_flow)))
	{
	  workflow->ptr = ptr;
	  return (ptr);
	}
    }
  return (NULL);
}

int remove_con_track_flow(t_workflow workflow)
{
  t_con_track *con_track_flow;

  con_track_flow = workflow.ptr;

  if (!con_track_flow)
    return (-1);

  if (con_track_flow->next && con_track_flow->prev)
    (con_track_flow->prev)->next = con_track_flow->next;
  else
    G_con_track[workflow.flow_hash] = NULL;

  G_cctde_infos.tcp_conn_alive_number -= 1;

  display_tcp_connection(workflow,2);

  free (con_track_flow);
  return (1);
}

/*
** Displaying functions
*/

void display_flow(FILE *stream,t_flow flow)
{
  display_infos (stream,"%s:%d->",ip_uint_to_str(flow.src),flow.sport);
  display_infos (stream,"%s:%d",  ip_uint_to_str(flow.dst),flow.dport);
}

void display_con_track_flows()
{
  int i;
  t_con_track *ptr;

  for(i=0; i<CON_HASH_SIZE; i++) 
    if (G_con_track[i] != NULL)
      for (ptr = G_con_track[i]; ptr != NULL; ptr = ptr->next)
	{
	  display_infos (stdout,"    # ");
	  display_flow(stdout,ptr->flow_infos);
	  display_infos (stdout," (%d / state : %d)\n",i,ptr->state);
	  display_infos (stdout,"      packets rx=%d tx=%d, bytes rx=%d tx=%d, uptime=%u\n",
		   ptr->packet_rx,ptr->packet_tx,ptr->bytes_rx,ptr->bytes_tx,
		   (u_int32_t) running_time(ptr->time_first_access.tv_sec,1));
	}
}

int display_tcp_connection(t_workflow workflow,int flag)
{
  char timestamp[26];
  t_storage *infos=NULL;

  if (G_cctde_infos.config.disp_tcp == 0 && G_cctde_infos.config.disp_cs != 1)
    return (0);

  if (workflow.ptr && (workflow.ptr)->infos)
    infos = (t_storage *) (workflow.ptr)->infos->data;

  bzero((char *) timestamp, 26);
  if (infos)
    ts_print((struct timeval *) & (infos->time_last_access), timestamp);
  else
    if (workflow.ptr)
      ts_print((struct timeval *) & (workflow.ptr)->time_first_access, timestamp);

  if (flag == 0 && G_cctde_infos.config.disp_tcp == 2)
    {
      display_infos (stdout,"[ %s] :     TCP packet [%d] : ",timestamp,workflow.flow_hash);
      display_flow(stdout,workflow.flow_infos);
      display_infos (stdout,"\n");
    }
  if (flag == 1 && G_cctde_infos.config.disp_tcp > 0)
    {
      display_infos (stdout,"[ %s] : SYN TCP packet [%d] : ",timestamp,workflow.flow_hash);
      display_flow(stdout,workflow.flow_infos);
      display_infos (stdout,"\n");
    }
  if (flag == 2 && G_cctde_infos.config.disp_tcp > 0)
    {
      display_infos (stdout,"[ %s] : FIN TCP packet [%d] : ",timestamp,workflow.flow_hash);
      display_flow(stdout,workflow.flow_infos);
      display_infos (stdout,"\n");
    }
  if (flag == 3 && G_cctde_infos.config.disp_cs == 1)
    {
      display_infos (stdout,"[ %s] : Desync TCP packet [%d] : ",timestamp,workflow.flow_hash);
      display_flow(stdout,workflow.flow_infos);
      display_infos (stdout,"\n");
    }

  return (1);
}

/*
** Managing TCP paquets as parts of TCP connections
*/

int manage_tcp_connection(Alertpkt alert,Packet *p)
{
  t_workflow workflow;
  /* pointer to stored connection tracking informations */
  t_con_track *ptr;
  /* pointer to stored informations element */
  t_list_elem *stored;
  t_storage *stored_infos, *stored_infos2;
  time_t com_uptime;

  /* Build the workflow */
  build_workflow(&workflow,p);

  if (G_cctde_infos.first_packet == 0 || 
      G_cctde_infos.first_packet > p->pkth->ts.tv_sec)
    G_cctde_infos.first_packet = p->pkth->ts.tv_sec;
  if (G_cctde_infos.last_packet < p->pkth->ts.tv_sec)
    G_cctde_infos.last_packet = p->pkth->ts.tv_sec;

  /* New connection */
  if ((p->tcph->th_flags & TH_NORESERVED) == (TH_SYN))
    {
      /* Only accept new connection for analyze if not_before and not_after flags
	 are not set in the command line */

	if (G_cctde_infos.config.not_before != 0 &&
	    p->pkth->ts.tv_sec < G_cctde_infos.config.not_before)
	  return (0);

	if (G_cctde_infos.config.not_after != 0 &&
	    p->pkth->ts.tv_sec > G_cctde_infos.config.not_after)
	  return (0);

      if ((ptr = add_con_track_flow(&workflow)) == NULL)
	return (-1);
      /* store more packet informations for this flow */
      ptr->packet_tx = 1;
      ptr->bytes_tx = ntohs(p->iph->ip_len) -
	((IP_HLEN(p->iph) << 2) + (TCP_OFFSET(p->tcph) << 2));
      ptr->time_first_access = p->pkth->ts;
      ptr->time_last_access = p->pkth->ts;
    }
  else /* Check if connection exists */
    {
      if ((ptr = find_con_track_flow(&workflow)) != NULL)
	{ 
	  if (FLOW_WAY(ptr,p) == 0)
	    {
	      ptr->packet_tx += 1;
	      ptr->bytes_tx += ntohs(p->iph->ip_len) -
		((IP_HLEN(p->iph) << 2) + (TCP_OFFSET(p->tcph) << 2));
	    }
	  else
	    {
	      ptr->packet_rx += 1;
	      ptr->bytes_rx += ntohs(p->iph->ip_len) -
		((IP_HLEN(p->iph) << 2) + (TCP_OFFSET(p->tcph) << 2));
	    }
	  ptr->time_last_access = p->pkth->ts;
	  
	  /* This packet belongs to a stored connection */
	  if (((p->tcph->th_flags & TH_FIN) == TH_FIN) ||
	      (p->tcph->th_flags & TH_RST) == TH_RST)
	    {
	      /* FIN or RESET */
	      if ((p->tcph->th_flags & TH_RST) == TH_RST)
		ptr->state = FLOW_STATE_TO_CLOSE;
	      else
		{
		  if (memcmp(&workflow.flow_infos,&(ptr->flow_infos),sizeof(t_flow)))
		    {
		      ptr->isn_c = ntohl(p->tcph->th_seq);
		      if (ntohl(p->tcph->th_ack) == ptr->isn_s+1)
			ptr->state = FLOW_STATE_CLOSING;
		    }
		  else
		    {
		      ptr->isn_s = ntohl(p->tcph->th_seq);
		      if (ntohl(p->tcph->th_ack) == ptr->isn_c+1)
			ptr->state = FLOW_STATE_CLOSING;
		    }
		}
	    }
	  else
	    {
	      /* First check if this ack is for a previous FIN */
	      if (ntohl(p->tcph->th_ack) == ptr->isn_c+1 ||
		  ntohl(p->tcph->th_ack) == ptr->isn_s+1)
		{
		  if (ptr->state == FLOW_STATE_CLOSING)
		    ptr->state = FLOW_STATE_TO_CLOSE;
		  else
		    ptr->state = FLOW_STATE_CLOSING;
		}
	      else
		ptr->state = FLOW_STATE_CONNECTED;
	    }
	}
      else
	{
	  if (G_cctde_infos.config.disp_cs == 1)
	    display_tcp_connection(workflow,3);
	  G_cctde_infos.desync_packets_number += 1;
	  return (0);
	}
    }

  /*
  ** Now we update and store informations
  */

  G_cctde_infos.packets_number += 1;
  G_cctde_infos.data_amount += ntohs(p->iph->ip_len) -
    ((IP_HLEN(p->iph) << 2) + (TCP_OFFSET(p->tcph) << 2));

  if (ptr)
    {
      if (ptr->state == FLOW_STATE_ON_CONNECT)
	display_tcp_connection(workflow,1);
      else
	display_tcp_connection(workflow,0);

      if ((stored = consider_timeline(workflow,alert,p)) == NULL)
	return (-1);
      else
	ptr->infos = (t_list_elem *) stored;
    }

  /*
  ** Do we have to remove this connection ?
  */

  if (ptr->state == FLOW_STATE_TO_CLOSE)
    {
      /* 
      ** Before we remove the current connection flow,
      ** we have to update fields such as the total com uptime..
      ** (in the client and server related view and in the main server view
      */
      com_uptime = ptr->time_last_access.tv_sec - ptr->time_first_access.tv_sec;

      stored_infos = (t_storage *) ptr->infos->data;
      stored_infos->com_uptime += com_uptime;
      bzero(&stored_infos->time_first_access,sizeof(struct timeval *));

      stored_infos2 = (t_storage *) stored_infos->sec_list_last_acceded->data;
      stored_infos2->com_uptime += com_uptime;
      bzero(&stored_infos2->time_first_access,sizeof(struct timeval *));

      stored_infos2 = (t_storage *) stored_infos2->server_infos->data;
      stored_infos2->com_uptime += com_uptime;
      bzero(&stored_infos2->time_first_access,sizeof(struct timeval *));

      // Last correlation for current flow before removing (last FIN)
      correlate_upcon(ptr);

      remove_con_track_flow(workflow);
    }

  return (1);
}
